@@ -26,6 +26,7 @@
 #endif
 
 Display *display;
+int display_fd;
 char *displayname;
 NPWindow windata;
 int flags;
@@ -100,7 +101,11 @@ static void place_window()
      XMapWindow(display, victim);
      XSync(display, FALSE);
 
-     maximize_window(flags, display, victim, w, h);
+     if (flags & H_FILL) {
+	  fill_window(display, victim, w, h);
+     } else if (flags & H_MAXASPECT) {
+	  max_aspect_window(display, victim, w, h);
+     }
 
      D("placewindow done\n");
 }
@@ -353,7 +358,9 @@ static int init_swallow()
 static void end_swallow()
 {
      if (flags & H_EVENT_SWALLOW)
+     {
 	  XSelectInput(display, DefaultRootWindow(display), 0);
+     }
 }
 
 /*****************************************************************************
@@ -387,12 +394,17 @@ static void handle_app(int pid)
      int swallow_done = 0;
      int status;
      int tmp;
+     fd_set fds;
+     struct timeval tv;
+
+     tv.tv_sec = 0;
+     tv.tv_usec = SWALLOW_WAIT_TIME;
 
      if ((flags & H_SWALLOW) || (flags & H_EVENT_SWALLOW))
      {
 	  for (tmp = 0; tmp < SWALLOW_LOOPS; tmp++)
 	  {
-	       if (!(flags & H_EXITS))
+	       if (!(flags & H_DAEMON))
 	       {
 		    if (waitpid(pid, &status, WNOHANG) > 0)
 		    {
@@ -408,7 +420,12 @@ static void handle_app(int pid)
 
 	       if (swallow_done) break;
 	       
-	       usleep(SWALLOW_WAIT_TIME);
+	       FD_ZERO(&fds);
+	       FD_SET(display_fd, &fds);
+
+	       D("select()\n");
+	       if (select(display_fd+1, &fds, NULL, NULL, &tv) == -1)
+		    exit(1);
 	  }
 	  end_swallow();
      }
@@ -422,7 +439,7 @@ static void handle_app(int pid)
 		    place_window();
 	  }
 
-	  if (flags & H_EXITS)
+	  if (flags & H_DAEMON)
 	       exit(0);
 
 	  D("Waiting for godot... %d\n",pid);
@@ -448,7 +465,7 @@ int main(int argc, char **argv)
 {
      D("Helper started.....\n");
 
-     if (argc < 7)
+     if (argc < 2)
      {
 	  fprintf(stderr,"MozPlugger version " VERSION " helper application.\n"
 		  "Written by Fredrik Hubinette and Louis Bavoil.\n" 
@@ -466,103 +483,45 @@ int main(int argc, char **argv)
 	    (int *)&windata.y,
 	    (int *)&windata.width,
 	    (int *)&windata.height);
-     file = argv[2];
-     winname = argv[3];
-     displayname = argv[4];
-     command = argv[5];
-     mimetype = argv[6];
 
-     if (!strlen(file)) file = 0;
-     if (!strlen(winname)) winname = 0;
-     if (!strlen(displayname)) displayname = 0;
+     command = argv[2];
+     winname = getenv("winname");
+     file = getenv("file");
 
-     D("HELPER: %s %s %s %s %s\n",
+     D("HELPER: %s %s %s %s\n",
        argv[0],
        argv[1],
        file,
-       displayname,
        command);
 
      while (repeats > 0)
      {
 	  char *argv[10];
-	  char buffer[16384];
-	  char *foo = buffer;
 	  int loops = 1;
 	  int pid;
 
 	  /* This application will use the $repeat variable */
 	  if (flags & H_REPEATCOUNT)
-	  {
 	       loops = repeats;
-	  }
 
 	  /* Expecting the application to loop */
 	  if (flags & H_LOOP)
-	  {
 	       loops = 0x7fffffff;
-	  }
-
-	  /* setup environment variable $file */
-	  if ((flags & H_MANY) && !(flags & H_REPEATCOUNT))
-	  {
-	       int e;
-	       snprintf(foo, sizeof(buffer), "file=%s", file);
-	       loops = MIN(repeats,10);
-	       for (e=0; e<loops; e++)
-	       {
-		    strcat(foo," ");
-		    strcat(foo,file);
-	       }
-	  } else {
-	       snprintf(foo, sizeof(buffer), "file=%s", file);
-	  }
-	  putenv(foo);
-	  foo+=strlen(foo)+1;
-
-	  /* setup environment variable $env */
-	  snprintf(foo, sizeof(buffer), "window=%ld", (long)WINDOW);
-	  putenv(foo);
-	  foo+=strlen(foo)+1;
-
-	  /* setup environment variable $repeat */
-	  snprintf(foo, sizeof(buffer), "repeat=%ld", (long)repeats);
-	  putenv(foo);
-	  foo+=strlen(foo)+1;
-
-	  /* setup environment variable $width */
-	  snprintf(foo, sizeof(buffer), "width=%ld", (long)windata.width);
-	  putenv(foo);
-	  foo+=strlen(foo)+1;
-
-	  /* setup environment variable $height */
-	  snprintf(foo, sizeof(buffer), "height=%ld", (long)windata.height);
-	  putenv(foo);
-	  foo+=strlen(foo)+1;
-
-	  /* setup environment variable $DISPLAY */
-	  if (displayname)
-	  {
-	       snprintf(foo, sizeof(buffer), "DISPLAY=%s", displayname);
-	       D("putenv(%s)\n",foo);
-	       putenv(foo);
-	       foo+=strlen(foo)+1;
-	  }
-
-	  /* setup environment variable $mimetype */
-	  if (mimetype)
-	  {
-	       snprintf(foo, sizeof(buffer), "mimetype=%s", mimetype);
-	       D("putenv(%s)\n",foo);
-	       putenv(foo);
-	       foo+=strlen(foo)+1;
-	  }
 
 	  /* Create command line */
-	  argv[0] = "/bin/sh";
-	  argv[1] = "-c";
-	  argv[2] = command;
-	  argv[3] = NULL;
+	  if (flags & H_CONTROLS)
+	  {
+	       argv[0] = getenv("controller");
+	       argv[1] = command;
+	       argv[2] = NULL;
+	  }
+	  else
+	  {
+	       argv[0] = "/bin/sh";
+	       argv[1] = "-c";
+	       argv[2] = command;
+	       argv[3] = NULL;
+	  }
 
 	  D("Execing %s (repeats=%d loops=%d)\n",command,repeats,loops);
 
@@ -570,7 +529,11 @@ int main(int argc, char **argv)
 	  {
 	       XSetErrorHandler(error_handler);
 
-	       if (!display) display = XOpenDisplay(displayname);
+	       if (!display)
+	       {
+		    display = XOpenDisplay(displayname);
+		    display_fd = ConnectionNumber(display);
+	       }
 
 	       if (init_swallow())
 	       {
